/*
    Trabajo Fin de Grado (TFG)
    Autor: Eduardo Bayón Alonso
    Tutor: Daniel Moríñigo Sotelo
    Se trata de la creación de un dispositivo con el fin de leer los valores que proporciona una célula solar calibrada, para convertirlos en W/m2 y obtener la irradiancia del lugar y posición en la cual se coloque. También se medirá la temperatura.
    Estos datos se irán almacenando en una tarjeta microSD gracias a la placa Intel Edison que proporciona esta facilidad con la inclusión interna de un adaptador.
*/


/*Inclusión de las librerías*/

#include <LiquidCrystal.h> //Librería para el manejo del LCD
//#include <SPI.h> // No necesita SPI ya que el adaptador esta conectado a traves de un controlador SD integrado
#include <SD.h> //Librería para la utilización de la microSD


/*Definición de los pines utilizados*/

//Definición de pines del LCD
const int rs = 2, en = 3, d4 = 7, d5 = 8, d6 = 12, d7 = 13;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

//Definición de los pines de sensor de irradiancia y temperatura
#define sensorIrr A0
#define sensorTemp1 A1
#define sensorTemp2 A2

//Definicion de los pines del LED RGB y LED indicador del almacenamiento de datos en SD
#define pinrojo 9
#define pinverde 10
#define pinazul 11
#define LED_indicador 13

//Definición de los pulsadores y de la retroiluminación de la pantalla LCD
#define pinpuls1 A3
#define pinpuls2 A4
#define pinpuls3 A5
#define pinluz 4


/*Declaración de variables globales*/

//Variables indicadoras del tipo de pulsación en el botón
#define PULSCORTA 1
#define PULSLARGA 0

bool luz = LOW; //Variable que indica la retroiluminación del LCD

char fecha[15], hora[15]; //Variables que almacenan el valor de la fecha y la hora en formato (d:m:a , h:m:s)

int puls; //Variable que guarda el valor de la pulsación (corta o larga)
int MODO = 1, MODO_ANT = 0; // Variables para indicar que mostrar por el LCD

int parametro = 0; //Variable que indica el parámetro dia, mes, año u hora, minuto, segundo en el que guardar el valor de estos
int valor[6] = {1, 1, 0, 0, 0, 0};  //Vector en el que se guardará el valor de la fecha y hora según el parámetro
int maxim[6] = {31, 12, 37, 23, 59, 59}; //Vector que indica los valores máximos permitidos para cada parámetro
int minim[6] = {1, 1, 0, 0, 0, 0}; //Vector que indica los valores mínimos permitidos para cada parámetro


int cont = 0; //Variable que cuenta el número de tomas por segundo
int irr = 0, temp = 0;  //Variables que acumulan el valor "en crudo" de las x tomas cada segundo
float irradiancia = 0, temperatura = 0; //Variables en las que almacenar los datos una vez convertidos a sus correspondientes en el SI

unsigned long millisLectura = 0; //Se declara la variable en la que se guarda el tiempo en la que se produce la impresión y el guardado de datos
unsigned long interv_global = 1000;  //Se declara la variable que establece el tiempo entre cada actualización de datos
unsigned long millisTomaDatos = 0;  //Se declara la variable en la que se guarda el tiempo en la que se produce la toma de datos
unsigned long interv_datos = 40;  //Se declara la variable que establece el tiempo entre cada toma de datos durante un segundo de lectura
unsigned long const_time = 0; //Se declara la variable de la diferencia de tiempo entre el introducido por el usuario y el de la Intel

time_t timestamp; //Variable en la que se almacenará en segundos la fecha que indique el usuario.

File archivoDatos; //Variable para el manejo de archivos en la tarjeta microSD


/*Creación de símbolos para el LCD*/
byte grado[8] = //Símbolo para los grados Celsius
{
  0b00001100,
  0b00010010,
  0b00010010,
  0b00001100,
  0b00000000,
  0b00000000,
  0b00000000,
  0b00000000
};

byte cuadrado[8] = //Símbolo de superíndice para los W/m2
{
  0b00001110,
  0b00000010,
  0b00001110,
  0b00001000,
  0b00001110,
  0b00000000,
  0b00000000,
  0b00000000
};

byte enie[8] = //Símbolo de la ñ
{
  0b00001110,
  0b00000000,
  0b00010110,
  0b00011001,
  0b00010001,
  0b00010001,
  0b00010001,
  0b00000000
};


/*************************************************************
  Función principal que se ejecuta al principio y una sola vez
*************************************************************/
void setup() {

  /*Modos en los que funciona cada pin*/
  //Entradas
  pinMode(sensorIrr, INPUT);
  pinMode(sensorTemp1, INPUT);
  pinMode(sensorTemp2, INPUT);
  pinMode(pinpuls1, INPUT_PULLUP); //Pulsador 1 . Activo a 0 por resistencia Pullup
  pinMode(pinpuls2, INPUT_PULLUP); //Pulsador 2 . Activo a 0 por resistencia Pullup
  pinMode(pinpuls3, INPUT_PULLUP); //Pulsador 3 . Activo a 0 por resistencia Pullup

  //Salidas
  pinMode(LED_indicador, OUTPUT);
  pinMode(pinluz, OUTPUT);
  pinMode(pinrojo, OUTPUT);
  pinMode(pinverde, OUTPUT);
  pinMode(pinazul, OUTPUT);

  Serial.begin(115200); //Se inicia la comunicación serie con tasa de transferencia 115200 baudios

  lcd.begin(16, 2); //Se inicia la comunicación con el LCD indicando el número de columnas y filas que utiliza

  //Se crean los caracteres definidos anteriormente
  lcd.createChar(0, grado);
  lcd.createChar(1, cuadrado);
  lcd.createChar(2, enie);

  lcd.clear(); //Se borra lo que hubiera en el display
  lcd.setCursor(0, 0);  //Se sitúa el cursor en la primera fila y primera columna
  digitalWrite(pinluz, HIGH); //Se pone a nivel alto la variable pinluz para encender la retroilumninación del LCD

  //Bucle comprobante de la inclusión de la microSD en el adaptador
  while (!SD.begin()) { //Realizar continuamente si la SD no se ha inicializado o no se ha introducido
    //Se mandan mensajes por monitor serie y LCD del error.
    Serial.println("Introduzca la tarjeta para empezar a medir.");
    lcd.print("Introduce la SD");
    lcd.setCursor(0, 1);
    lcd.print(" para comenzar. ");
    delay(2000);  //Se espera 2 segundos para el siguiente ciclo
  }
  //En caso de haberse iniciado correctamente se manda el mensaje
  Serial.println("SD correctamente inicializada");
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("SD inicializada.");
  delay(1000);  //Se espera un segundo para que se lea correctamente el mensaje por LCD

  establecer_fecha(); //Llamada a la función que permite establecer la fecha mediante botones
}


/**********************************************
  Función principal que se ejecuta cíclicamente
**********************************************/
void loop() {

  if (millis() - millisTomaDatos  >= interv_datos) {  //Si han pasado 40ms(intervalo entre toma de datos) desde la última toma de datos
    millisTomaDatos = millis();

    if (digitalRead(pinpuls1) == LOW) {   //Si el usuario pulsa el botón 1, se guarda un 2 en la variable MODO
      MODO = 2;
      MODO_ANT = MODO;  //Se guarda la variable MODO en MODO_ANT
    }
    if (digitalRead(pinpuls2) == LOW) {   //Si el usuario pulsa el botón 2, se guarda un 3 en la variable MODO
      MODO = 3;
      MODO_ANT = MODO;  //Se guarda la variable MODO en MODO_ANT
    }
    if (digitalRead(pinpuls3) == LOW) {   //Si el usuario pulsa el botón 3, se guarda un 4 en la variable MODO
      MODO = 4;
    }
    
    if (millis() - millisLectura >= interv_global) {  //Si ha pasado 1 segundo (intervalo de actualización de datos) desde la última actualización
      millisLectura = millis();

      actualizar_fecha(); //Se llama a la función actualizar_fecha

      conversion_sensores(); //Se llama a la función conversion_sensores

      LED_RGB();  //Se llama a la función LED_RGB que iluminará un led según el valor de la irradiancia

      switch (MODO) { //Según el modo que escoja el usuario, se llamará a una función u otra
        case 2:
          mostrar_irradiancia();  //Si el usuario escoge ese modo, se llama a la función mostrar_iradiancia que imprimirá por LCD y monitor serie la irradiancia
          break;
        case 3:
          mostrar_temperatura();  //Si el usuario escoge ese modo, se llama a la función mostrar_temperatura que imprimirá por LCD y monitor serie la temperatura
          break;
        case 4:
          onOff();  //Si el usuario escoge ese modo, se llama a la función onOff que apagará o encenderá el LCD
          MODO = MODO_ANT; //Vuelve al modo en el que estaba anteriormente una vez realizada la función con el fin de que el LCD no quede vacío
          break;
        default: //En caso de que el modo fuera otro, no se ejecuta nada y sale.
          break;
      }

      if (cont >= 1) //En caso de que contador sea mayor o igual que 1 se llama a la función almacenamiento para que en la primera ejecución los valores de irradiancia y temperatura no sean 0 al guardarlos.
        almacenamiento();

      //Cada vez que pasa el segundo se resetean los valores para volver a acumularlos en el siguiente segundo y poder tratarlos externamente
      irr = 0;
      temp = 0;
      cont = 0;
      Serial.println("*****************************************************");
    }

    irr = irr + analogRead(sensorIrr);      //Sumatorio de los valores de la irradiancia
    temp =  temp + (analogRead(sensorTemp1) - analogRead(sensorTemp2)); //Sumatorio de los valores de la temperatura

    /*Serial.print("cont ");  //quitar
      Serial.print(cont);
      Serial.print("    ");
      Serial.print("irr ");
      Serial.print(irr);
      Serial.print("    ");
      Serial.print("temp ");
      Serial.println(temp);*/
    cont ++;  //Se incrementa contador para controlar el primer dato guardado
  }
}


/***********************************************************************************************************************************************************
  Función para que el usuario establezca la fecha y hora actual, la cual se indicará en el fichero donde se guardarán los datos de irradiancia y temperatura
***********************************************************************************************************************************************************/
void establecer_fecha() {

  Serial.println("Establezca fecha (d/m/a h:m:s)");
  lcd.print("Establezca fecha");
  info_parametro(parametro, valor); //Se llama a la función info_parametro para que inicialmente en el LCD y el monitor serie, una vez inicializada la SD aparezca el parámetro y su valor correspondiente

  while (MODO == 1) { //Se realiza el bucle continuamente mientras el MODO sea 1 (Bucle de comprobación de botones)
    //INICIO PULSADOR 1
    if (digitalRead(pinpuls1) == LOW) { //En caso de que se haya apretado el pulsador 1...
      puls = leer_pulsador(pinpuls1); //Asigna a la variable puls, el valor que devuelve la función leer_pulsador del pulsador 1
      if (puls == PULSCORTA) {  //Si lo que se ha devuelto es un 0 (pulsación corta), se incrementa 1 el parámetro
        parametro++;
        if (parametro > 5)  //Si el párametro es mayor que 5 (segundo), vuelve a 0 (día)
          parametro = 0;
        info_parametro(parametro, valor); //Se llama a la función que actualiza la fecha y hora a mostrar
        puls = 2; //Se ajusta el valor de puls para que no interfiera en el resto de los if
      }
      if (puls == PULSLARGA) {  //Si lo que se ha devuelto es un 1 (pulsación larga), se resetea el valor de los parámetros y se vuelve al parámetro inicial (dia)
        Serial.print("Reseteando...");
        lcd.clear();
        lcd.print("Reseteando...");
        //Se asignan los valores iniciales
        parametro = 0;
        valor[0] = 1;
        valor[1] = 1;
        for (int i = 2; i <= 5; i++) {
          valor[i] = 0;
        }
        delay(1200); //Se deja un tiempo para leer en el LCD "Reseteando"
        info_parametro(parametro, valor); //Se llama a la función para actualizar el LCD y mostrar por monitor serie la fecha y hora
      }
      delay(200); //Se deja un tiempo de 200 milisegundos entre 2 posibles lecturas de pulsación
    }
    //FIN PULSADOR 1

    //INICIO PULSADOR 2
    if (digitalRead(pinpuls2) == LOW) { //En caso de que se haya apretado el pulsador 2...
      puls = leer_pulsador(pinpuls2); //Asigna a la variable puls, el valor que devuelve la función leer_pulsador del pulsador 2
      if (puls == PULSCORTA) {  //Si la pulsación ha sido corta (menor de 2 segundos)
        valor[parametro]++;   //Aumenta la variable valor del parametro actual en 1
        if (valor[parametro] > maxim[parametro])  //En caso de que se exceda el máximo valor de ese parámetro, se establece el mínimo
          valor[parametro] = minim[parametro];
        info_parametro(parametro, valor); //Se llama a la función para actualizar el LCD y mostrar por monitor serie la fecha y hora
        puls = 2; //Se ajusta el valor de puls para que no interfiera en el resto de los if
      }
      delay(200); //Se deja un tiempo de 200 milisegundos entre 2 posibles lecturas de pulsación
    }
    //FIN PULSADOR 2

    //INICIO PULSADOR 3
    if (digitalRead(pinpuls3) == LOW) { //En caso de que se haya apretado el pulsador 3...
      puls = leer_pulsador(pinpuls3); //Asigna a la variable puls, el valor que devuelve la función leer_pulsador del pulsador 3
      if (puls == PULSCORTA) {  //Si la pulsación ha sido corta
        valor[parametro]--; //Disminuye la variable valor del parametro actual en 1
        if (valor[parametro] < minim[parametro])  //En caso de que se disminuir del mínimo valor de ese parámetro, se establece el máximo
          valor[parametro] = maxim[parametro];
        info_parametro(parametro, valor); //Se llama a la función para actualizar el LCD y mostrar por monitor serie la fecha y hora
        puls = 2; //Se ajusta el valor de puls para que no interfiera en el resto de los if
      }
      if (puls == PULSLARGA) {  //En caso de ser una pulsación larga
        Serial.println("Guardando valores..."); //Se imprime el mensaje por el monitor serie y por el LCD
        Serial.println("");
        lcd.clear();
        lcd.print("Guardando los");
        lcd.setCursor(0, 1);
        lcd.print("valores...");
        delay(1200);  //Se deja un tiempo para leer corectamente el LCD
        //puls = 2; //Se ajusta el valor de puls para que no interfiera en el resto de los if
        MODO = 0; //Se establece MODO == 0 para salir del bucle while (ya que se ha establecido la fecha)
        //Se borra el LCD y se imprime el menú desde el que elige el usuario qué desea que se muestre por el monitor serie y el LCD
        Serial.println("Pulse el botón 1 para mostrar irradiancia, 2 para mostrar temperatura y 3 para apagar o encender el LCD");
        Serial.println("");
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("MODO:  1-IRRAD");
        lcd.setCursor(0, 1);
        lcd.print("2-TEMP 3-ON/OFF");
      }
      delay(200); //Se deja un tiempo de 200 milisegundos entre 2 posibles lecturas de pulsación
    }
    //FIN PULSADOR 3
  }

  time_t tiempo_intel = time(NULL);   //Se crea un time_t (tipo entero) con la fecha inicial que tiene la Intel en ese momento
  time_t tiempo_introducido = estruct_a_time_t();  //Se crea otro time_t con la fecha introducida por el usuario
  const_time = (tiempo_introducido - tiempo_intel); //Se calcula la diferencia entre la fecha introducida y la actual de la Intel en una variable que será constante

  /***********************************************************/
  Serial.println("################################");
  Serial.println(tiempo_intel);
  Serial.print(asctime(localtime(&tiempo_intel)));
  Serial.println("********************************");
  Serial.println(tiempo_introducido);
  Serial.print(asctime(localtime(&tiempo_introducido)));
  Serial.println("################################");
  /***********************************************************/
}


/******************************************************************************
  Función que determina el tipo de pulsación que se ha realizado en los botones
******************************************************************************/
int leer_pulsador(int pin) {

  unsigned long tiempopulsado = millis(); //Se almacena el tiempo actual (cuando se ha pulsado)
  //PULSACIÓN LARGA
  if (pin == pinpuls1 || pin == pinpuls3) { //Si se han pulsado los botones 1 o 3 (que permiten la pulsación larga)...
    while (digitalRead(pin) == LOW) { //Se realiza el bucle mientras esté a nivel bajo la lectura de pulsación (mientras esté pulsado)
      if ((millis() - tiempopulsado) >= 2000) { //Si desde que se ha pulsado el botón hasta ahora han pasado 2 segundos, la función devuelve 1
        return 0;
      }
    }
  }
  //PULSACIÓN CORTA
  if (pin == pinpuls1 || pin == pinpuls2 || pin == pinpuls3) { // Si se han pulsado 1, 2 o 3 (menos de 2 segundos)
    return 1;
  }
}


/*********************************************************************************************************************************************
  Función que imprime por el monitor serie y el LCD el parámetro actual, así como su valor durante la ejecución de la función establecer_fecha
*********************************************************************************************************************************************/
void info_parametro(int parametro, int valor[]) {

  lcd.clear();
  lcd.print("Establezca fecha");
  lcd.setCursor(2, 1);
  //Según sea un parámetro u otro, imprimirá lo correspondiente
  switch (parametro) {
    case 0:
      Serial.print("Dia:  ");
      lcd.print("Dia:");
      break;
    case 1:
      Serial.print("Mes:  ");
      lcd.print("Mes:");
      break;
    case 2:
      Serial.print("Año:  ");
      lcd.print("A");
      lcd.write(byte(2)); //Se escribe el símbolo creado para el LCD
      lcd.print("o:");
      break;
    case 3:
      Serial.print("Hora: ");
      lcd.print("Hora:");
      break;
    case 4:
      Serial.print("Minuto: ");
      lcd.print("Minuto:");
      break;
    case 5:
      Serial.print("Segundo:  ");
      lcd.print("Segundo:");
      break;
  }
  lcd.setCursor(12, 1);
  if (valor[parametro] < 10) {  //Si el valor es menor que 10, se añade un cero delante para que quede estético
    lcd.print("0");
    Serial.print("0");
  }
  lcd.print(valor[parametro]);
  Serial.println(valor[parametro]);
}


/*************************************************************************************************************************************************
   Función que guarda en una estructura del estándar de linux la fecha y hora con los valores introducidos por el usuario, y lo convierte a time_t
*************************************************************************************************************************************************/
time_t estruct_a_time_t() {

  time_t tiempo;  //Se define variable en la que se guardará en segundos, el tiempo introducido por el usuario
  struct tm timeinfo; //Se define la estructura en la que se guardan los valores introducidos por el usuario en dia-mes-año hora:minuto:segundo

  timeinfo.tm_year = valor[2] + 100 , timeinfo.tm_mon = valor[1] - 1, timeinfo.tm_mday = valor[0],
           timeinfo.tm_hour = valor[3], timeinfo.tm_min = valor[4], timeinfo.tm_sec = valor[5]; //Se asignan los valores a cada parámetro de la estructura
  tiempo = mktime(&timeinfo); //Convierte el valor de las variables de la estructura de formato separado a su correspondiente en formato condensado (en segundos)
  return tiempo; //Se devuelve el valor tiempo
}


/******************************************************************************************************
   Función que se encarga de la actualización de la fecha en cada pasada del bucle en donde se la llama
******************************************************************************************************/
void actualizar_fecha() {

  time_t fecha_actual = time(&fecha_actual);  //Se crea otra variable local con la fecha actual de la tarjeta Intel
  timestamp = fecha_actual + const_time;  //Se asigna a la variable timestamp global el valor de fecha_actual y la diferencia entre la introducida por el usuario y la de la Intel dando el valor final del tiempo
  struct tm* tiempo_struct = localtime(&timestamp);  //Se crea una estructura para dar formato a los segundos del timestamp con días, meses...

  // Se da formato a los valores contenidos en la estructura colocándolos en los array de caractéres fecha y hora
  strftime(hora, sizeof hora, "%H:%M:%S", tiempo_struct);
  strftime(fecha, sizeof fecha, "%d-%m-%Y", tiempo_struct);
}


/*********************************************************************************************************************
   Función que procede a la conversión de los datos de irradiancia y temperatura acumulados, para su posterior muestra
*********************************************************************************************************************/
int conversion_sensores() {

  float valorSensorIrr = irr / 25.0;  //Se guarda en la variable de irradiancia el valor promediado de la irradiancia acumulada
  float voltaje0 = (valorSensorIrr * 5000) / 1024.0; // Se convierte la lectura analógica (Rango 0 - 1023) a voltaje (0 - 5000mV)
  irradiancia = voltaje0 * 1000 / 400.0;   // Se convierte a W/m2 por medio del datasheet de la placa solar  que indica: 100mV igual a 1Kw/m2 = 1000W/m2
  //Debido al circuito de amplificación que aumenta la ganancia 4 veces, se divide entre 400 en vez de 100

  float valorSensorTemp = temp / 25.0;  //Se guarda en la variable de temperatura el valor promediado de la temperatura acumulada
  float voltaje1 = (valorSensorTemp * 5000) / 1024.0; // Se convierte la lectura analógica (Rango 0 - 1023) a voltaje (0 - 5000mV)
  temperatura = voltaje1 / 30.0;  //Se convierte a °C ya que según datasheet LM35, la ganancia es de 10 --> 10mV es 1 ºC
  //Debido al circuito de amplificación que aumenta la ganancia 3 veces, se divide entre 30 en vez de 10
}


/************************************************************************************************************************
   Función que se encarga de crear un fichero en la SD con el nombre de la fecha introducida por el usuario, y guardar la fecha, el valor de la irradiancia y la temperatura en crudo, para su posterior procesamiento en un programa externo.
************************************************************************************************************************/
void almacenamiento() {

  //Creación del nombre del archivo para guardar los datos
  char fichero[20];
  strcpy(fichero, fecha); //Se copia en el vector de caractéres "fichero" el valor de la cadena fecha
  strcat(fichero, ".txt");  //Se concatena fichero con el texto ".txt" para la creación del archivo posterior

  archivoDatos = SD.open(fichero, FILE_WRITE);  //Se abre fichero o se crea en caso de no existir en modo lectura-escritura
  if (archivoDatos) {   //Si se ha podido abrir el fichero...
    digitalWrite(LED_indicador, HIGH);  //Se enciende el LED indicador
    archivoDatos.print(timestamp);  //Se escribe en el fichero el tiempo en formato condensado
    archivoDatos.print(",");
    archivoDatos.print(irr); //Se escribe la lectura de la irradiancia acumulada (las 10 tomas) en el fichero
    archivoDatos.print(",");
    archivoDatos.print(temp);  //Se escribe la lectura de la temperatura acumulada (las 10 tomas) en el fichero
    archivoDatos.println("");
    archivoDatos.close(); //Se cierra el fichero para que se guarden correctamente los datos
  }
  else {  //En caso de no haberse podido abrir el fichero, se envía un mensaje de error
    Serial.println("Error de apertura del archivo, reinicie dispositivo.");
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Error apertura." );
    lcd.setCursor(0, 1);
    lcd.print("Reinicie aparato" );
  }
  digitalWrite(LED_indicador, LOW); //Se apaga el LED indicando la finalización del archivado de los datos
}


/****************************************************************************************************************
   Función que iluminará un LED RGB con diferentes colores según sea la medición de la irradiancia en ese momento
****************************************************************************************************************/
void LED_RGB() {

  if (irradiancia < 1750) {  //Los intervalos están establecidos de 1750 en 1750. Color rojo
    digitalWrite(pinrojo, 100);
    digitalWrite(pinazul, 0);
    digitalWrite(pinverde, 0);
  }  else if (irradiancia < 1750 && irradiancia >= 3500) {  //Color amarillo
    digitalWrite(pinverde, 100);
    digitalWrite(pinazul, 100);
    digitalWrite(pinrojo, 0);
  } else if (irradiancia < 3500 && irradiancia >= 5250) {  //Color verde
    digitalWrite(pinverde, 100);
    digitalWrite(pinazul, 0);
    digitalWrite(pinrojo, 0);
  } else if (irradiancia < 5250 && irradiancia >= 7000) {  //Color morado
    digitalWrite(pinverde, 0);
    digitalWrite(pinazul, 100);
    digitalWrite(pinrojo, 100);
  } else if (irradiancia >= 7000) {   //Color azul
    digitalWrite(pinazul, 100);
    digitalWrite(pinverde, 0);
    digitalWrite(pinrojo, 0);
  }
}


/*************************************************************************************
   Función que mostrará la irradiancia del momento tanto por el monitor serie como por el display LCD, así como la fecha de la toma de esos datos por el monitor serie
*************************************************************************************/
void mostrar_irradiancia() {

  Serial.print("Fecha y hora: ");
  Serial.print(fecha);
  Serial.print("  ");
  Serial.println(hora);
  //Serial.print("  TimeStamp:  ");
  //Serial.println(timestamp);
  Serial.print("El valor de la irradiancia es: ");
  Serial.print(irradiancia);
  Serial.println(" W/m2");
  //Serial.print("  Irradiancia en crudo:  ");
  //Serial.println(valorSensorIrr);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(" La irradiancia " );
  lcd.setCursor(0, 1);
  lcd.print(" es: " );
  lcd.print(irradiancia);
  lcd.print("W/m");
  lcd.write(byte(1));
}


/*************************************************************************************
   Función que mostrará la temperatura del momento tanto por el monitor serie como por el display LCD, así como la fecha de la toma de esos datos por el monitor serie
*************************************************************************************/
void mostrar_temperatura() {

  Serial.print("Fecha y hora: ");
  Serial.print(fecha);
  Serial.print("  ");
  Serial.println(hora);
  Serial.print("El valor de la temperatura es: ");
  Serial.print(temperatura);
  Serial.println("°C");
  //Serial.print("  Temperatura en crudo:  ");
  //Serial.println(valorSensorTemp);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(" La temperatura " );
  lcd.setCursor(0, 1);
  lcd.print("  es: " );
  lcd.print(temperatura);
  lcd.write(byte(0));
  lcd.print("C");
}


/*****************************************************************************************************
   Función que se encargará de apagar o encender el LCD según se pulse el tercer botón del dispositivo
*****************************************************************************************************/
void onOff() {

  if (luz == 1) { //En caso de estar encendido
    lcd.noDisplay();  //El LCD deja de mostrar lo que está mostrando
    luz = !luz;   //Cambia el valor de la variable de retroiluminación
    digitalWrite(pinluz, luz);  //El LCD apaga la retroiluminación
  } else if (luz == 0) {  //Si no, si se encuentra apagado
    lcd.display();  //El LCD muestra de nuevo lo que estaba mostrando
    luz = !luz; //Cambia el valor de la variable de retroiluminación
    digitalWrite(pinluz, luz);  //El LCD enciende la retroiluminación
  }
  Serial.println(luz);
}
